diff --git a/src/bin/pg_dump/compress_io.c b/src/bin/pg_dump/compress_io.c
index 7a18e77..6320ec8 100644
--- a/src/bin/pg_dump/compress_io.c
+++ b/src/bin/pg_dump/compress_io.c
@@ -490,7 +490,8 @@ cfopen_read(const char *path, const char *mode)
 /*
  * Open a file for writing. 'path' indicates the path name, and 'mode' must
  * be a filemode as accepted by fopen() and gzopen() that indicates writing
- * ("w", "wb", "a", or "ab").
+ * ("w", "wb", "a", or "ab"), gzopen() being completed by the compression
+ * level requested by caller.
  *
  * If 'compression' is non-zero, a gzip compressed stream is opened, and
  * and 'compression' indicates the compression level used. The ".gz" suffix
@@ -509,7 +510,7 @@ cfopen_write(const char *path, const char *mode, int compression)
 		char	   *fname;
 
 		fname = psprintf("%s.gz", path);
-		fp = cfopen(fname, mode, 1);
+		fp = cfopen(fname, mode, compression);
 		free(fname);
 #else
 		exit_horribly(modulename, "not built with zlib support\n");
@@ -531,7 +532,11 @@ cfopen(const char *path, const char *mode, int compression)
 	if (compression != 0)
 	{
 #ifdef HAVE_LIBZ
-		fp->compressedfp = gzopen(path, mode);
+		char mode_compression[16];
+
+		snprintf(mode_compression, sizeof(mode_compression), "%s%d",
+				 mode, compression);
+		fp->compressedfp = gzopen(path, mode_compression);
 		fp->uncompressedfp = NULL;
 		if (fp->compressedfp == NULL)
 		{
